"""
An event provider: DNS queries
"""

from dns import message
from cxutil import flags,cmask
from socket import socket,AF_INET,SOCK_DGRAM
from command import command

class dns_socket(object):
	"""
	DNS server socket
	"""
	fd = None
	sock = None
	signal = None

	def __init__(self,signal,bind,port):
		self.signal = signal
		self.sock = socket(AF_INET,SOCK_DGRAM)
		self.sock.bind((bind,port))
		self.fd = self.sock.fileno()

	def read(self):
		s = self.sock.recvfrom(65535)
		b = {}
		
		b["src"] = s[1][0]
		b["sport"] = s[1][1]
		b["query"] = message.from_wire(s[0])

		self.signal("dns.query",b)

class dnsq (command):
	"""
	An event provider: DNS queries
	"""
	mask = flags.Esoteric

	parameters = {
		"$1": {
			"type": "word",
			"name": "bind",
			"description": "An address to bind to (default: all interfaces)",
			"default": "",
		},
		"port": {
			"type": "int",
			"description": "A port to bind to (default: 53) (currently only UDP is supported)",
			"default": 53,
		},
	}

	def init(self,opts):
		if not opts.call_mask & cmask.Immediate:
			self.s = dns_socket(opts.signal,opts.envl.bind,int(opts.envl.port))
			opts.esoteric.bus.put("monitor",("register",self.s.fd,self.s.read))

	def postUp(self,opts):
		pass
